/*
 * PositionInfoForm.java
 *
 * Created on 23. November 2007, 17:55
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

import de.gjlay.mobile.localize.L;
import de.gjlay.mobile.localize.LanguageListener;
import de.gjlay.mobile.localize.Localizer;
import de.gjlay.mobile.ui.Message;
import de.gjlay.mobile.ui.MessageListener;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

/**
 *
 * @author Georg-Johann Lay
 */
class PositionInfoForm
    extends Form
    implements BOARD_CONST, CommandListener, LanguageListener, MessageListener
{
    private MIDlet midlet;
    private Localizer localizer;

    private static final int T_CASTLE = 1;
    private static final int T_EP = 2;

    private MovesChoiceGroup choiceWhite;
    private MovesChoiceGroup choiceBlack;
    private MovesChoiceGroup choiceEP;

    private Command backCommand;
    private Command okCommand;
    private Board b;

    private static PositionInfoForm instance;

    private static final int TAG_ERROR = 0;

    /**
     * Creates a new instance of PositionInfoForm
     */
    PositionInfoForm (MIDlet midlet, Localizer localizer)
    {
        super ("");
        this.midlet = midlet;
        this.localizer = localizer;
        init ();
    }

    static PositionInfoForm GetInstance (MIDlet midlet, Localizer localizer)
    {
        return
            instance = new PositionInfoForm (midlet, localizer);
    }

    private void init()
    {
        //languageChanged();
        setCommandListener (this);
        localizer.addLanguageListener (this);
    }

    public void commandAction (Command command, Displayable displayable)
    {
        if (backCommand == command)
            PositionCanvas.Show (false);

        if (okCommand == command)
        {
            setBoardInfo();
            b.save ("sample");
            PlayCanvas.Show (b);
        }
    }

    private String getErrorMessage ()
    {
        String error = L._("T_ERROR")+":\n";

        if (null == b.king[WHITE])
            return error+L._("T_MISSING_WHITE_KING");

        if (null == b.king[BLACK])
            return error+L._("T_MISSING_BLACK_KING");

        if (b.isCheck (1-b.mover))
            return error+L._("T_IMPOSSIBLE_CHECK");

        return null;
    }

    private void setBoardInfo()
    {
        choiceWhite.setCastle (b);
        choiceBlack.setCastle (b);
        choiceEP.setEP (b);
    }

    static void ProcessBoard (Board b)
    {
        instance.processBoard (b);
    }

    private void processBoard (Board b)
    {
        this.b = b;

        String msg = getErrorMessage();
        if (null != msg)
        {
            Message.Show (this, TAG_ERROR, Message.ERROR_EVENT, msg);
            return;
        }

        b.startsWithCheck = b.isCheck (b.mover);

        deleteAll();

        choiceWhite.setBoard (b, WHITE);
        choiceBlack.setBoard (b, BLACK);
        choiceEP.setBoard (b, 1-b.mover);

        if (BLACK == b.mover)
        {
            addGroup (choiceBlack, false);
            addGroup (choiceWhite, true);
        }

        if (WHITE == b.mover)
        {
            addGroup (choiceWhite, false);
            addGroup (choiceBlack, true);
        }
/*
        if (choiceWhite.size() > 0
                || (choiceEP.size() > 1 && BLACK == b.mover))
        {
            append (choiceWhite);
            if (choiceEP.size() > 1 && BLACK == b.mover)
                append (choiceEP);
        }

        if (choiceBlack.size() > 0
                || (choiceEP.size() > 1 && WHITE == b.mover))
        {
            append (choiceBlack);
            if (choiceEP.size() > 1 && WHITE == b.mover)
                append (choiceEP);
        }
*/
        if (0 != size())
            Display.getDisplay (midlet).setCurrent (this);
        else
            commandAction (okCommand, null);
    }

    private void addGroup (ChoiceGroup choice, boolean addEP)
    {
        if (choice.size() > 0
                || (choiceEP.size() > 1 && addEP))
        {
            append (choice);
            if (choiceEP.size() > 1 && addEP)
                append (choiceEP);
        }
    }

    private class MovesChoiceGroup
            extends ChoiceGroup
    {
        private int color;
        private int base, step, type;

        private MovesChoiceGroup (String label, int type)
        {
            super (label, (T_EP == type) ? Choice.EXCLUSIVE : Choice.MULTIPLE);
            this.type  = type;
        }

        private void setBoard (Board b, int color)
        {
            this.deleteAll();

            base = (color == WHITE) ? 0 : 7;
            step = (color == WHITE) ? 1 : -1;
            this.color = color;

            if (type == T_CASTLE)
            {
                if (may_0_0 (b))
                    append (Move.STR_0_0, true);

                if (may_0_0_0 (b))
                    append (Move.STR_0_0_0, true);
            }

            if (type == T_EP)
            {
                for (int x = 0; x < 8; x++)
                {
                    int y = base + 3*step;

                    if (!mayEP (b, x, y))
                        continue;

                    String s = ""
                            + Field.Get (x, y-2*step)
                            + Move.STR_MOVES
                            + Field.Get (x, y);

                    append (s, false);
                }
                append (L._("T_OTHER_MOVE"), true);
            }
        }

        private boolean mayEP (Board b, int x, int y)
        {
            return b.p[x][y]        == (PAWN | color)
                && b.p[x][y-step]   == EMPTY
                && b.p[x][y-step*2] == EMPTY
                && ((Board.Ok (x-1,y) && b.p[x-1][y]  == (PAWN | (1-color)))
                 || (Board.Ok (x+1,y) && b.p[x+1][y]  == (PAWN | (1-color))))
                && !((Board.Ok (x-1,y) && b.p[x-1][y-step] == (KING | (1-color)))
                 ||  (Board.Ok (x+1,y) && b.p[x+1][y-step] == (KING | (1-color))));
        }

        private void append (String label, boolean selected)
        {
            append (label, null);
            setSelectedIndex (size()-1, selected);
        }

        private boolean may_0_0 (Board b)
        {
            return b.p[4][base] == (KING | color)
                    && b.p[7][base] == (ROOK | color);
        }

        private boolean may_0_0_0 (Board b)
        {
            return b.p[4][base] == (KING | color)
                    && b.p[0][base] == (ROOK | color);
        }

        private void setEP (Board b)
        {
            b.lastPawn2x = -1;

            int i = getSelectedIndex();

            if (i != size()-1)
                b.lastPawn2x = getString(i).charAt(0)-'a';
        }

        private void setCastle (Board b)
        {
            b.mayCastle &= ~Board.M_0_0[color] & ~Board.M_0_0_0[color];

            for (int i=0; i < size(); i++)
                if (isSelected(i))
                    b.mayCastle |= (Move.STR_0_0 == getString(i) ? Board.M_0_0 : Board.M_0_0_0)[color];
        }
    }

    public void languageChanged (String locale)
    {
        setTitle (L._("T_ALLOWED_MOVES"));
        choiceWhite = new MovesChoiceGroup (L._("T_WHITE"), T_CASTLE);
        choiceBlack = new MovesChoiceGroup (L._("T_BLACK"), T_CASTLE);
        choiceEP = new MovesChoiceGroup (L._("T_LAST_MOVE"), T_EP);

        removeCommand (backCommand);
        removeCommand (okCommand);
        addCommand (backCommand = new Command (L._("T_BACK"), Command.BACK, 1));
        addCommand (okCommand   = new Command (L._("T_OK"), Command.OK, 1));
    }

    public void messageReturned (int id, int tag)
    {
        if (TAG_ERROR == tag)
        {
            // resume
            PositionCanvas.Show (false);
        }
    }

}
